--
-- Excavators Tracks Position for FS19
-- This mod shows you current position of tracks against the cab of your excavator.
-- @author:    	kenny456 (kenny456@seznam.cz)
-- @history:	v1.0 - 2017-11-14 - 
-- 				v1.1 - 2019-03-12 - conversion to FS19
-- 								  - added option for resize excavators tracks icon
-- 								  - added option for hide / show F1 help menu for mod
--								  - added option for activate / deactivate mod function for current vehicle or global
-- 								  - added xmlConfig file for global settings
-- 
ExcavatorsTracksPosition = {};
ExcavatorsTracksPosition.modDirectory  = g_currentModDirectory
ExcavatorsTracksPosition.confDir = getUserProfileAppPath().. "modsSettings/ExcavatorsTracksPosition/";

function ExcavatorsTracksPosition.prerequisitesPresent(specializations)
	return true
end

function ExcavatorsTracksPosition.initSpecialization()
end

function ExcavatorsTracksPosition.registerOverwrittenFunctions(vehicleType)
end

function ExcavatorsTracksPosition.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "toggleReverse", ExcavatorsTracksPosition.toggleReverse)
	SpecializationUtil.registerFunction(vehicleType, "loadFromXml", ExcavatorsTracksPosition.loadFromXml)
	SpecializationUtil.registerFunction(vehicleType, "saveToXml", ExcavatorsTracksPosition.saveToXml)
	SpecializationUtil.registerFunction(vehicleType, "updateValues", ExcavatorsTracksPosition.updateValues)
	SpecializationUtil.registerFunction(vehicleType, "activateEtp", ExcavatorsTracksPosition.activateEtp)
end

function ExcavatorsTracksPosition.registerEvents(vehicleType)
end

function ExcavatorsTracksPosition:registerActionEventsPlayer()
end

function ExcavatorsTracksPosition:registerActionEventsMenu()
end

function ExcavatorsTracksPosition:onRegisterActionEvents(isSelected, isOnActiveVehicle)
	local spec = self.spec_ExcavatorsTracksPosition
	
	if g_dedicatedServerInfo ~= nil then
		return
	end
	if spec.event_IDs == nil then
		spec.event_IDs = {}
	end
	if isOnActiveVehicle and self:getIsControlled() then
		local actions_RC1 = { InputAction.ETP_REVERSE_DIRECTION, InputAction.ETP_TOGGLE_SIZE, InputAction.ETP_TOGGLE_HELP, InputAction.ETP_TOGGLE, InputAction.ETP_TOGGLE_ALL }

		for _,actionName in pairs(actions_RC1) do
			local _, eventID = g_inputBinding:registerActionEvent(actionName, self, ExcavatorsTracksPosition.actionCallback, true, true, false, spec.hasCab)
			spec.event_IDs[actionName] = eventID
			if g_inputBinding ~= nil and g_inputBinding.events ~= nil and g_inputBinding.events[eventID] ~= nil then
				g_inputBinding.events[eventID].displayPriority = 2
				if actionName == InputAction.ETP_TOGGLE_HELP then
					g_inputBinding:setActionEventTextVisibility(eventID, true)
				elseif actionName == InputAction.ETP_TOGGLE or actionName == InputAction.ETP_TOGGLE_ALL then
					g_inputBinding:setActionEventTextVisibility(eventID, false)
				else
					g_inputBinding:setActionEventTextVisibility(eventID, ExcavatorsTracksPosition.showHelp)
				end
			end
		end
	end
end

function ExcavatorsTracksPosition.registerEventListeners(vehicleType)
	for _,n in pairs( { "onLoad", "onPostLoad", "saveToXMLFile", "onUpdate", "onUpdateTick", "onDraw", "onRegisterActionEvents", "registerActionEventsPlayer", "saveToXml", "loadFromXml", "onReadStream", "onWriteStream", "onEnterVehicle" } ) do
		SpecializationUtil.registerEventListener(vehicleType, n, ExcavatorsTracksPosition)
	end
end

function ExcavatorsTracksPosition:onLoad(savegame)
	local spec = self.spec_ExcavatorsTracksPosition
	spec.event_IDs = {}
end

function ExcavatorsTracksPosition:onPostLoad(savegame)
	local spec = self.spec_ExcavatorsTracksPosition
	
	ExcavatorsTracksPosition.showHelp = true
	ExcavatorsTracksPosition.etpActive = true
	ExcavatorsTracksPosition.size = 1
	spec.ui = g_gameSettings.uiScale
	spec.maxSize = 1.4
	spec.minSize = 0.6
	spec.hasCab = false
	spec.reverse = false
	spec.etpActive = true
	spec.showRPMHudOverlay = createImageOverlay(ExcavatorsTracksPosition.modDirectory .. "excav.dds")
	spec.showRPMHudOverlay2 = createImageOverlay(ExcavatorsTracksPosition.modDirectory .. "tracks.dds")
	spec.showRPMHudOverlay3 = createImageOverlay(ExcavatorsTracksPosition.modDirectory .. "Info_hud.dds")
    setOverlayColor(spec.showRPMHudOverlay3, 0, 0, 0, 0.75)
	
	if self.spec_cylindered.movingTools ~= 0 and self.spec_cylindered.movingTools ~= nil and self.spec_cylindered.movingTools[1] ~= nil then
		for _,tool in pairs(self.spec_cylindered.movingTools) do
			if(tool.axisActionIndex == "AXIS_CRANE_ARM" or tool.axisActionIndex == "AXIS_FRONTLOADER_ARM" or tool.axisActionIndex == "AXIS_CRANE_TOOL" or tool.axisActionIndex == "AXIS_FRONTLOADER_TOOL") and ((tool.rotMax == nil and tool.rotMin == nil) or tool.rotMax == tool.rotMin * -1) then
				spec.hasCab = true
				spec.toolAxis = tool
			end
		end
	end
	
	if savegame ~= nil then
		local xmlFile = savegame.xmlFile
		local key = savegame.key ..".ExcavatorsTracksPosition"
		spec.reverse = Utils.getNoNil(getXMLBool(xmlFile, key.."#reverse"), spec.reverse);
		self:toggleReverse(spec.reverse, true)
	end
	
	local configFile = ExcavatorsTracksPosition.confDir .. "ExcavatorsTracksPositionConfig.xml";
	if fileExists(configFile) then
		ExcavatorsTracksPosition.configXml = loadXMLFile("ExcavatorsTracksPosition_XML", configFile);
		ExcavatorsTracksPosition.loadFromXml(self, self:getFullName());
	else
		createFolder(getUserProfileAppPath().. "modsSettings/");
		createFolder(ExcavatorsTracksPosition.confDir);
		ExcavatorsTracksPosition.configXml = createXMLFile("ExcavatorsTracksPosition_XML", configFile, "ExcavatorsTracksPositionConfig");
		ExcavatorsTracksPosition.saveToXml(self);
		saveXMLFile(ExcavatorsTracksPosition.configXml);
	end
	ExcavatorsTracksPosition.updateValues(self)
end
function ExcavatorsTracksPosition:updateValues()
	local spec = self.spec_ExcavatorsTracksPosition
	
	spec.sizeHud = ExcavatorsTracksPosition.size * spec.ui
	spec.positionX = 0.872;
	spec.positionY = 0.174;
	
	spec.showWidth3 = 0.06 * spec.sizeHud;
	spec.showHeight3 = 0.174 * spec.sizeHud;
	spec.showPosX3 = spec.positionX - (spec.showWidth3 * 1)
    spec.showPosY3 = spec.positionY - 0.145;
	
	spec.showWidth = 0.11 * spec.sizeHud;
	spec.showHeight = 0.16 * spec.sizeHud;
	spec.showPosX = spec.showPosX3 - ((spec.showWidth - spec.showWidth3) / 2)
    spec.showPosY = spec.positionY - 0.125;
	
	spec.showWidth2 = 0.12 * spec.sizeHud;
	spec.showHeight2 = 0.16 * spec.sizeHud;
	spec.showPosX2 = spec.showPosX3 - ((spec.showWidth2 - spec.showWidth3) / 2)
    spec.showPosY2 = spec.positionY - 0.125;
end
function ExcavatorsTracksPosition:saveToXMLFile(xmlFile, key)
	local spec = self.spec_ExcavatorsTracksPosition
	setXMLBool(xmlFile, key.."#reverse", spec.reverse)
end

function ExcavatorsTracksPosition:loadFromXml(vehicle)
	local spec = self.spec_ExcavatorsTracksPosition
    if ExcavatorsTracksPosition.configXml == nil then
        return;
    end
	if getXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpShowHelp") ~= nil then
		ExcavatorsTracksPosition.showHelp = getXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpShowHelp")
	end;
	if getXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveGlobal") ~= nil then
		ExcavatorsTracksPosition.etpActive = getXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveGlobal")
	end
	if getXMLFloat(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpHudSize") ~= nil then
		ExcavatorsTracksPosition.size = getXMLFloat(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpHudSize")
	end
	if vehicle ~= nil then
		local key = "ExcavatorsTracksPositionConfig.etpActiveVehicles"
		local i = 0
		while true do
			local name = getXMLString(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i).."#name");
			if name ~= nil then
				if name == vehicle then
					spec.etpActive = getXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i));
					break
				end
			else
				break
			end
			i = i + 1
		end
	end
end

function ExcavatorsTracksPosition:saveToXml(vehicle, active)
	local spec = self.spec_ExcavatorsTracksPosition
	
    if ExcavatorsTracksPosition.configXml == nil then
        return;
    end
	setXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpShowHelp", ExcavatorsTracksPosition.showHelp);
	setXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveGlobal", ExcavatorsTracksPosition.etpActive);
	setXMLFloat(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpHudSize", ExcavatorsTracksPosition.size);
	if vehicle ~= nil and active ~= nil then
		local key = "ExcavatorsTracksPositionConfig.etpActiveVehicles"
		local i = 0
		while true do
			local name = getXMLString(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i).."#name");
			if name ~= nil then
				if name == vehicle then
					setXMLString(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
					setXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i), active);
					break
				end
			else
				setXMLString(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
				setXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i), active);
				break
			end
			i = i + 1
			local name = key .. string.format(".vehicle(%d)", i)
			if not hasXMLProperty(ExcavatorsTracksPosition.configXml, name) then
				setXMLString(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i).."#name", vehicle);
				setXMLBool(ExcavatorsTracksPosition.configXml, "ExcavatorsTracksPositionConfig.etpActiveVehicles"..string.format(".vehicle(%d)", i), active);
				break
			end
		end
	end
    saveXMLFile(ExcavatorsTracksPosition.configXml);
end

function ExcavatorsTracksPosition:onUpdate(dt)
	local spec = self.spec_ExcavatorsTracksPosition
	
	if self:getIsActive() then
		if spec.hasCab == true and spec.toolAxis.curRot[2] ~= nil then
			setOverlayRotation(spec.showRPMHudOverlay2, (spec.toolAxis.curRot[2] * -1) - math.rad(spec.reverse and 180 or 0), spec.showWidth2/2, spec.showHeight2/3.8)
			if (spec.event_IDs ~= nil) and g_dedicatedServerInfo == nil then
				for i,eventID in pairs(spec.event_IDs) do
					if i ~= InputAction.ETP_TOGGLE and i ~= InputAction.ETP_TOGGLE_ALL then
						g_inputBinding:setActionEventActive(eventID, spec.etpActive and ExcavatorsTracksPosition.etpActive)
					else
						g_inputBinding:setActionEventActive(eventID, true)
					end
					if i ~= InputAction.ETP_TOGGLE_HELP and i ~= InputAction.ETP_TOGGLE and i ~= InputAction.ETP_TOGGLE_ALL then
						g_inputBinding:setActionEventTextVisibility(eventID, ExcavatorsTracksPosition.showHelp)
					end
					if ExcavatorsTracksPosition.showHelp then
						g_inputBinding:setActionEventText(spec.event_IDs['ETP_TOGGLE_HELP'], g_i18n:getText('ETP_HELP_HIDE'))
					else
						g_inputBinding:setActionEventText(spec.event_IDs['ETP_TOGGLE_HELP'], g_i18n:getText('ETP_HELP_SHOW'))
					end
					g_inputBinding:setActionEventText(spec.event_IDs['ETP_TOGGLE_SIZE'], g_i18n:getText('input_ETP_TOGGLE_SIZE')..' : '..string.format("%.1f",ExcavatorsTracksPosition.size)..'/'..spec.maxSize )
				end
			end
		end
	end
end
function ExcavatorsTracksPosition:onUpdateTick(dt)
end

function ExcavatorsTracksPosition:onDraw()
	local spec = self.spec_ExcavatorsTracksPosition
	
	if self:getIsActive() and g_dedicatedServerInfo == nil and self.isClient then
		if spec.hasCab == true and spec.etpActive and ExcavatorsTracksPosition.etpActive then
			renderOverlay(spec.showRPMHudOverlay3, spec.showPosX3, spec.showPosY3, spec.showWidth3, spec.showHeight3)
			renderOverlay(spec.showRPMHudOverlay2, spec.showPosX2, spec.showPosY2, spec.showWidth2, spec.showHeight2)
			renderOverlay(spec.showRPMHudOverlay, spec.showPosX, spec.showPosY, spec.showWidth, spec.showHeight)
		end
	end
end

function ExcavatorsTracksPosition:actionCallback(actionName, keyStatus, arg4, arg5, arg6)
	local spec = self.spec_ExcavatorsTracksPosition
	
	if self:getIsActive() then
		if keyStatus > 0 then
			if actionName == 'ETP_REVERSE_DIRECTION' then
				self:toggleReverse(not spec.reverse)
			elseif actionName == 'ETP_TOGGLE_SIZE' then
				if ExcavatorsTracksPosition.size + 0.2 <= spec.maxSize then
					ExcavatorsTracksPosition.size = ExcavatorsTracksPosition.size + 0.2
				else
					ExcavatorsTracksPosition.size = spec.minSize
				end
				ExcavatorsTracksPosition.updateValues(self)
				ExcavatorsTracksPosition.saveToXml(self);
			elseif actionName == 'ETP_TOGGLE_HELP' then
				ExcavatorsTracksPosition.showHelp = not ExcavatorsTracksPosition.showHelp
				ExcavatorsTracksPosition.saveToXml(self);
			elseif actionName == 'ETP_TOGGLE' then
				if ExcavatorsTracksPosition.etpActive == false then
					g_currentMission:showBlinkingWarning("FIRST ACTIVATE EXCAVATORS TRACK POSITION GLOBAL (CTRL + ALT + X)")
				else
					spec.etpActive = not spec.etpActive
					self:activateEtp(spec.etpActive, 'THIS')
					ExcavatorsTracksPosition.saveToXml(self, self:getFullName(), spec.etpActive)
				end
			elseif actionName == 'ETP_TOGGLE_ALL' then
				ExcavatorsTracksPosition.etpActive = not ExcavatorsTracksPosition.etpActive
				self:activateEtp(ExcavatorsTracksPosition.etpActive, 'ALL')
				ExcavatorsTracksPosition.saveToXml(self);
			end
		elseif keyStatus == 0 then
		end
	end
end
function ExcavatorsTracksPosition:activateEtp(status, mode)
	local spec = self.spec_ExcavatorsTracksPosition
	
	if mode == 'THIS' then
		mode = self:getFullName()
	end
	local text = 'ACTIVATED'
	if not status then
		text = 'DEACTIVATED'
	end
	g_currentMission:showBlinkingWarning("EXCAVATORS TRACK POSITION FUNCTIONS "..tostring(text).." FOR "..tostring(mode).." VEHICLES", 2000);
end
function ExcavatorsTracksPosition:onEnterVehicle()
	local spec = self.spec_ExcavatorsTracksPosition
	
	ExcavatorsTracksPosition.updateValues(self)
	ExcavatorsTracksPosition.loadFromXml(self, self:getFullName())
end
function ExcavatorsTracksPosition:toggleReverse(reverse, noEventSend)
	local spec = self.spec_ExcavatorsTracksPosition
	
	ExcavatorsTracksPositionToggleEvent.sendEvent(self, reverse, noEventSend)
	spec.reverse = reverse
end
function ExcavatorsTracksPosition:onReadStream(streamId, connection)
	local spec = self.spec_ExcavatorsTracksPosition
	
	local reverse = streamReadFloat32(streamId);
	if reverse ~= nil  then
		self:toggleReverse(reverse, true)
	end
end;

function ExcavatorsTracksPosition:onWriteStream(streamId, connection)
	local spec = self.spec_ExcavatorsTracksPosition
	
	streamWriteFloat32(streamId, spec.reverse);
end;

ExcavatorsTracksPositionToggleEvent = {};
ExcavatorsTracksPositionToggleEvent_mt = Class(ExcavatorsTracksPositionToggleEvent, Event);

InitEventClass(ExcavatorsTracksPositionToggleEvent, "ExcavatorsTracksPositionToggleEvent");

function ExcavatorsTracksPositionToggleEvent:emptyNew()
    local self = Event:new(ExcavatorsTracksPositionToggleEvent_mt);
    self.className="ExcavatorsTracksPositionToggleEvent";
    return self;
end;

function ExcavatorsTracksPositionToggleEvent:new(object, reverse)
	local self = ExcavatorsTracksPositionToggleEvent:emptyNew()
	self.object = object;
	self.reverse = reverse;
	return self;
end;

function ExcavatorsTracksPositionToggleEvent:readStream(streamId, connection)
	self.object = NetworkUtil.readNodeObject(streamId);
	self.reverse = streamReadFloat32(streamId);
    self:run(connection);
end;

function ExcavatorsTracksPositionToggleEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.object);
	streamWriteFloat32(streamId, self.reverse);
end;

function ExcavatorsTracksPositionToggleEvent:run(connection)
	if self.object ~= nil then
		self.object:toggleReverse(self.reverse, true);
	end
	if not connection:getIsServer() then
		g_server:broadcastEvent(ExcavatorsTracksPositionToggleEvent:new(self.object, self.reverse), nil, connection, self.object);
	end;
end;

function ExcavatorsTracksPositionToggleEvent.sendEvent(vehicle, reverse, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(ExcavatorsTracksPositionToggleEvent:new(vehicle, reverse), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(ExcavatorsTracksPositionToggleEvent:new(vehicle, reverse));
		end;
	end;
end;